import { Injectable } from '@nestjs/common';
import { TrpcService } from '@server/trpc/trpc.service';
import { CourseService } from './course.service';
import { z, ZodType } from 'zod';
import { Prisma } from '@nice/common';

const CourseCreateArgsSchema: ZodType<Prisma.CourseCreateArgs> = z.any();
const CourseUpdateArgsSchema: ZodType<Prisma.CourseUpdateArgs> = z.any();
const CourseFindManyArgsSchema: ZodType<Prisma.CourseFindManyArgs> = z.any();
const CourseWhereUniqueInputSchema: ZodType<Prisma.CourseWhereUniqueInput> = z.any();

@Injectable()
export class CourseRouter {
  constructor(
    private readonly trpc: TrpcService,
    private readonly courseService: CourseService,
  ) {}

  router = this.trpc.router({
    create: this.trpc.protectProcedure
      .input(CourseCreateArgsSchema)
      .mutation(async ({ input }) => {
        return await this.courseService.create(input);
      }),

    update: this.trpc.protectProcedure
      .input(CourseUpdateArgsSchema)
      .mutation(async ({ input }) => {
        return await this.courseService.update(input);
      }),
      softDeleteByIds: this.trpc.protectProcedure
      .input(
        z.object({
          ids: z.array(z.string()),
          data: CourseUpdateArgsSchema.nullish(),
        }),
      )
      .mutation(async ({ input }) => {
        return await this.courseService.softDeleteByIds(input.ids);
      }),
    delete: this.trpc.protectProcedure
      .input(CourseWhereUniqueInputSchema)
      .mutation(async ({ input }) => {
        return await this.courseService.delete({ where: input });
      }),

    findMany: this.trpc.procedure
      .input(CourseFindManyArgsSchema)
      .query(async ({ input }) => {
        return await this.courseService.findMany(input);
      }),


    findWithScores: this.trpc.procedure
      .input(z.number())
      .query(async ({ input }) => {
        return await this.courseService.findWithScores(input);
      }),

    search: this.trpc.procedure
      .input(z.object({
        name: z.string().optional(),
        courseId: z.number().optional(),
        limit: z.number().optional()
      }))
      .query(async ({ input }) => {
        return await this.courseService.search(input);
      }),
  });
}