import { Injectable } from '@nestjs/common';
import { TrpcService } from '@server/trpc/trpc.service';
import { ScoreService } from './score.service';
import { z, ZodType } from 'zod';
import { Prisma } from '@nice/common';

const ScoreCreateArgsSchema: ZodType<Prisma.ScoreCreateArgs> = z.any();
const ScoreUpdateArgsSchema: ZodType<Prisma.ScoreUpdateArgs> = z.any();
const ScoreFindManyArgsSchema: ZodType<Prisma.ScoreFindManyArgs> = z.any();
const ScoreWhereUniqueInputSchema: ZodType<Prisma.ScoreWhereUniqueInput> = z.any();

@Injectable()
export class ScoreRouter {
  constructor(
    private readonly trpc: TrpcService,
    private readonly scoreService: ScoreService,
  ) {}

  router = this.trpc.router({
    create: this.trpc.protectProcedure
      .input(ScoreCreateArgsSchema)
      .mutation(async ({ input }) => {
        return await this.scoreService.create(input);
      }),

    update: this.trpc.protectProcedure
      .input(ScoreUpdateArgsSchema)
      .mutation(async ({ input }) => {
        return await this.scoreService.update(input);
      }),
      softDeleteByIds: this.trpc.protectProcedure
      .input(
        z.object({
          ids: z.array(z.string()),
          data: ScoreUpdateArgsSchema.nullish(),
        }),
      )
      .mutation(async ({ input }) => {
        return await this.scoreService.softDeleteByIds(input.ids);
      }),
    delete: this.trpc.protectProcedure
      .input(ScoreWhereUniqueInputSchema)
      .mutation(async ({ input }) => {
        return await this.scoreService.delete({ where: input });
      }),

    findMany: this.trpc.procedure
      .input(ScoreFindManyArgsSchema)
      .query(async ({ input }) => {
        return await this.scoreService.findMany(input);
      }),

    findByClass: this.trpc.procedure
      .input(z.number())
      .query(async ({ input }) => {
        return await this.scoreService.findByClass(input);
      }),

    findWithScores: this.trpc.procedure
      .input(z.number())
      .query(async ({ input }) => {
        return await this.scoreService.findWithScores(input);
      }),

    search: this.trpc.procedure
      .input(z.object({
        name: z.string().optional(),
        classId: z.number().optional(),
        limit: z.number().optional()
      }))
      .query(async ({ input }) => {
        return await this.scoreService.search(input);
      }),
  });
}