import { Injectable } from '@nestjs/common';
import {
  db,
  Prisma,
  UserProfile,
  VisitType,
  Post,
  PostType,
  RolePerms,
  ResPerm,
  ObjectType,
  CourseMethodSchema,
} from '@nice/common';
import { MessageService } from '../message/message.service';
import { BaseService } from '../base/base.service';
import { DepartmentService } from '../department/department.service';
import EventBus, { CrudOperation } from '@server/utils/event-bus';
import { BaseTreeService } from '../base/base.tree.service';
import { z } from 'zod';
@Injectable()
export class ScoreService extends BaseTreeService<Prisma.ScoreDelegate> {
  constructor(
    private readonly messageService: MessageService,
    private readonly departmentService: DepartmentService,
  ) {
    super(db, ObjectType.SCORE, 'scoreAncestry', true);
  }
  async create(
    args: Prisma.ScoreCreateArgs,
    params?: { staff?: UserProfile; tx?: Prisma.TransactionClient },
  ) {
    const result = await super.create(args);
    EventBus.emit('dataChanged', {
      type: ObjectType.SCORE,
      operation: CrudOperation.CREATED,
      data: result,
    });
    return result;
  }
  async update(args: Prisma.ScoreUpdateArgs, staff?: UserProfile) {
    const result = await super.update(args);
    EventBus.emit('dataChanged', {
      type: ObjectType.SCORE,
      operation: CrudOperation.UPDATED,
      data: result,
    });
    return result;
  }
  async findByClass(scoreId: number) {
    return await db.score.findMany({
      where: { id: scoreId },
      include: {
        student: true,
        course: true,
      }
    });
  }
    /**
   * 查找学生并包含成绩信息
   */
    async findWithScores(studentId: number) {
      return await db.student.findUnique({
        where: { id: studentId },
        include: {
          class: true,
          scores: {
            include: {
              course: true
            }
          }
        }
      });
    }
      /**
   * 搜索学生
   */
  async search(params: { name?: string, classId?: number, limit?: number }) {
    const { name, classId, limit = 30 } = params;
    
    return await db.student.findMany({
      where: {
        ...(name && { name: { contains: name } }),
        ...(classId && { classId })
      },
      include: {
        class: true
      },
      take: limit
    });
  }
  private emitDataChangedEvent(data: any, operation: CrudOperation) {
    EventBus.emit("dataChanged", {
      type: this.objectType,
      operation,
      data,
    });
  }
}
