import { Injectable } from '@nestjs/common';
import {
  db,
  Prisma,
  UserProfile,
  VisitType,
  Post,
  PostType,
  RolePerms,
  ResPerm,
  ObjectType,
  CourseMethodSchema,
} from '@nice/common';
import { MessageService } from '../message/message.service';
import { BaseService } from '../base/base.service';
import { DepartmentService } from '../department/department.service';
import EventBus, { CrudOperation } from '@server/utils/event-bus';
import { BaseTreeService } from '../base/base.tree.service';
import { z } from 'zod';
@Injectable()
export class StudentService extends BaseTreeService<Prisma.StudentDelegate> {
  constructor(
    private readonly messageService: MessageService,
    private readonly departmentService: DepartmentService,
  ) {
    super(db, ObjectType.STUDENT, 'studentAncestry', true);
  }
  async create(
    args: Prisma.StudentCreateArgs,
    params?: { staff?: UserProfile; tx?: Prisma.TransactionClient },
  ) {
    const result = await db.student.create(args);
    EventBus.emit('dataChanged', {
      type: ObjectType.STUDENT,
      operation: CrudOperation.CREATED,
      data: result,
    });
    return result;
  }
  async update(args: Prisma.StudentUpdateArgs, staff?: UserProfile) {
    const result = await db.student.update(args);
    EventBus.emit('dataChanged', {
      type: ObjectType.STUDENT,
      operation: CrudOperation.UPDATED,
      data: result,
    });
    return result;
  }
  async findByClass(classId: number) {
    return await db.student.findMany({
      where: { classId },
      include: {
        class: true,
        scores: {
          include: {
            course: true
          }
        }
      }
    });
  }
    /**
   * 查找学生并包含成绩信息
   */
    async findWithScores(studentId: number) {
      return await db.student.findUnique({
        where: { id: studentId },
        include: {
          class: true,
          scores: {
            include: {
              course: true
            }
          }
        }
      });
    }
      /**
   * 搜索学生
   */
  async search(params: { name?: string, classId?: number, limit?: number }) {
    const { name, classId, limit = 30 } = params;
    
    return await db.student.findMany({
      where: {
        ...(name && { name: { contains: name } }),
      },
      include: {
        class: true
      },
      take: limit
    });
  }
  private emitDataChangedEvent(data: any, operation: CrudOperation) {
    EventBus.emit("dataChanged", {
      type: this.objectType,
      operation,
      data,
    });
  }
}
