'use client';
import React, { useState, useEffect } from 'react';
import Header from '../component/Header';
import StudentSearch from '../component/StudentSearch';
import ScoreForm from '../component/ScoreForm';
import ScoreTable from '../component/ScoreTable';
import { api } from '@nice/client';
const StudentPage = () => {
    const [scores, setScores] = useState([]);
    const [filteredScores, setFilteredScores] = useState([]);
    const [students, setStudents] = useState([]);
    const [courses, setCourses] = useState([]);
    const [loading, setLoading] = useState(true);
    const [currentPage, setCurrentPage] = useState(1);
    const [editingScore, setEditingScore] = useState(null);
    const [searchParams, setSearchParams] = useState({ term: '', type: 'name' });
    const itemsPerPage = 5;
    const { 
        data: studentsData = [], 
        isLoading, 
        error } = api.student.findMany.useQuery({
        select: {
            id: true,
            name: true,
            classId: true,
        }
    });
    console.log('学生',studentsData);
    // 模拟API调用获取数据
    const {data:coursesData} = api.course.findMany.useQuery({
        select: {
            id: true,
            name: true,
            scores: true,
        }
    });
    console.log('课程',coursesData);
    const {data:scoresData} = api.score.findMany.useQuery({
        select: {
            id: true,
            studentId: true,
            courseId: true,
            score: true,
        }
    });
    console.log('成绩',scoresData);
    useEffect(() => {
        const fetchData = async () => {
            try {
                // 这里应该替换为实际的API调用
                setTimeout(async () => {
                    setStudents(studentsData);
                    setCourses(coursesData);
                    setScores(scoresData);
                    setFilteredScores(scoresData);
                    setLoading(false);
                }, 1000);
            } catch (error) {
                console.error('获取数据失败', error);
                setLoading(false);
            }
        };
        fetchData();
    }, []);
    // 处理搜索
    const handleSearch = (term, type) => {
        setSearchParams({ term, type });
        if (!term.trim()) {
            setFilteredScores(scores);
            return;
        }
        let filtered = [];
        const lowerTerm = term.toLowerCase();
        switch (type) {
            case 'name':
                filtered = scores.filter(score => {
                    const student = students.find(s => s.id === score.studentId);
                    return student && student.name.toLowerCase().includes(lowerTerm);
                });
                break;
            case 'class':
                filtered = scores.filter(score => {
                    const student = students.find(s => s.id === score.studentId);
                    const className = student ? student.classId : null;
                    return className && className.toString().includes(lowerTerm);
                });
                break;
            case 'course':
                filtered = scores.filter(score => {
                    const course = courses.find(c => c.id === score.courseId);
                    return course && course.name.toLowerCase().includes(lowerTerm);
                });
                break;
            default:
                filtered = scores;
        }
        setFilteredScores(filtered);
        setCurrentPage(1); // 重置到第一页
    };
    // // 处理添加/编辑成绩
    const handleScoreSubmit = (formData) => {
        if (editingScore) {
            // 更新成绩
            const updatedScores = scoresData.map(score =>
                score.id === editingScore.id ? { ...score, ...formData } : score
            );
            setScores(updatedScores);
            setFilteredScores(
                filteredScores.map(score =>
                    score.id === editingScore.id ? { ...score, ...formData } : score
                )
            );
            setEditingScore(null);
        } else {
            // 添加新成绩
            const newScore = {
                id: scoresData.length + 1, // 实际项目中应该由后端生成
                ...formData
            };
            setScores([...scoresData, newScore]);
            setFilteredScores([...filteredScores, newScore]);
        }
    };
    // // 处理编辑成绩
    const handleEditScore = (score) => {
        setEditingScore(score);
        // 滚动到表单位置
        window.scrollTo({ top: 0, behavior: 'smooth' });
    };

    // 处理删除成绩
    const handleDeleteScore = (id) => {
            const updatedScores = scoresData.filter(score => score.id !== id);
            setScores(updatedScores);
            setFilteredScores(filteredScores.filter(score => score.id !== id));
    };

    // 分页相关
    const indexOfLastItem = currentPage * itemsPerPage;
    const indexOfFirstItem = indexOfLastItem - itemsPerPage;
    const currentItems = Array.isArray(filteredScores) 
        ? filteredScores.slice(indexOfFirstItem, indexOfLastItem) 
        : [];
    const totalPages = Math.ceil((filteredScores?.length || 0) / itemsPerPage);
    const handlePageChange = (pageNumber) => {
        setCurrentPage(pageNumber);
    };

    if (loading) {
        return (
            <div className="min-h-screen bg-gray-100">
                <Header />
                <div className="container mx-auto px-4 py-8">
                    <div className="flex justify-center items-center h-64">
                        <div className="text-xl text-gray-600">加载中...</div>
                    </div>
                </div>
            </div>
        );
    }

    return (
        <div className="min-h-screen bg-gray-100">
            <Header />
            <div className="container mx-auto px-4 py-8">
                <div className="grid grid-cols-1 lg:grid-cols-3 gap-6">
                    <div className="lg:col-span-1">
                        <ScoreForm onSubmit={handleScoreSubmit} editingScore={editingScore} students={studentsData} courses={coursesData} />
                    </div>
                    <div className="lg:col-span-2">
                        <StudentSearch onSearch={handleSearch} />
                        <ScoreTable
                            scores={scoresData || []}
                            students={studentsData || []}
                            courses={coursesData || []}
                            currentPage={currentPage}
                            totalPages={totalPages}
                            onPageChange={handlePageChange}
                            onEdit={handleEditScore}
                            onDelete={handleDeleteScore}
                        />
                    </div>
                </div>
            </div>
        </div>
    );
};

export default StudentPage;
